import os

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

# MPS environments
VALID_ENVIRONMENTS = ("dev", "qa", "prod")

# MPS Bucket used by ingest process
MPS_BUCKET_NAME = "edu.harvard.huit.lts.mps.{account}-{space}-{environment}"

# MPS API endpoints - dev (older network restricted ALBs)
MPS_INGEST_ENDPOINT_PRIVATE = (
    "https://mps-admin-{environment}.lib.harvard.edu/admin/ingest/initialize"
)
MPS_JOBSTATUS_ENDPOINT_PRIVATE = (
    "https://mps-admin-{environment}.lib.harvard.edu/admin/ingest/jobstatus/"
)

# MPS API endpoints - QA and prod (new public ALBs)
MPS_INGEST_ENDPOINT_QA = "https://mps-ingest-qa.lib.harvard.edu/admin/ingest/initialize"
MPS_JOBSTATUS_ENDPOINT_QA = (
    "https://mps-ingest-qa.lib.harvard.edu/admin/ingest/jobstatus/"
)
MPS_INGEST_ENDPOINT_PROD = "https://mps-ingest.lib.harvard.edu/admin/ingest/initialize"
MPS_JOBSTATUS_ENDPOINT_PROD = (
    "https://mps-ingest.lib.harvard.edu/admin/ingest/jobstatus/"
)

# Base URL for images
MPS_ASSET_BASE_URL = (
    "https://mps-{environment}.lib.harvard.edu/assets/images/{namespace}:"
)
MPS_ASSET_BASE_URL_PROD = "https://mps.lib.harvard.edu/assets/images/{namespace}:"

# Base URL for manifests
# The format should be URN-3:{namespace}:{manifest_name}:MANIFEST:{PreziVersion}
MPS_MANIFEST_BASE_URL = "https://nrs-{environment}.lib.harvard.edu/URN-3:{namespace}:"
MPS_MANIFEST_BASE_URL_PROD = "https://nrs.lib.harvard.edu/URN-3:{namespace}:"

# Service status endpoints
MPS_DEV_INGEST_SERVICE_STATUS = (
    "https://mps-admin-dev.lib.harvard.edu/admin/ingest/version"
)
MPS_QA_INGEST_SERVICE_STATUS = (
    "https://mps-ingest-qa.lib.harvard.edu/admin/ingest/version"
)
MPS_PROD_INGEST_SERVICE_STATUS = (
    "https://mps-ingest.lib.harvard.edu/admin/ingest/version"
)



import json
import os

import jsonschema
from IIIFpres import iiifpapi3

iiifpapi3.INVALID_URI_CHARACTERS = iiifpapi3.INVALID_URI_CHARACTERS.replace(
    ":", ""
)  # See https://github.com/giacomomarchioro/pyIIIFpres/issues/11


def createManifest(
    base_url: str,
    labels: list,  # of dicts or strings
    canvases: list,
    providers: list = [],
    behaviors: list = ["paged"],
    default_lang: str = "en",
    service_type: str = "ImageService2",  # Image API level - see https://iiif.io/api/presentation/3.0/#service
    service_profile: str = "level2",  # Compliance level - see https://iiif.io/api/image/2.1/compliance/ and https://iiif.io/api/image/3.0/compliance/
    # service_id: str = "https://mps-qa.lib.harvard.edu/assets/images/", # Currently set in annotation.id; used for annotation.body.id and service.id; this is MPS rather than NRS; could be hoisted to here
    manifest_metadata: list = None,
    rights: str = None,
    required_statement: list = None,
    summary: list = None,  # can also be str
    thumbnails: list = None,
) -> iiifpapi3.Manifest():
    """Creates and validates a IIIF manifest"""

    manifest = iiifpapi3.Manifest()
    iiifpapi3.BASE_URL = base_url + "/"
    manifest.set_id(objid=base_url)

    for label in labels:
        if isinstance(label, str):
            manifest.add_label(default_lang, label)
        else:
            manifest.add_label(label.get("lang", default_lang), label["label"])
    for behavior in behaviors:
        manifest.add_behavior(behavior)

    if manifest_metadata:
        for m in manifest_metadata:
            manifest.add_metadata(
                label=m["label"],
                value=m["value"],
                language_l=m.get("label_lang", default_lang),
                language_v=m.get("value_lang", default_lang),
            )

    if rights:
        manifest.set_rights(rights)

    if required_statement:
        for m in required_statement:
            manifest.add_requiredStatement(
                label=m["label"],
                value=m["value"],
                language_l=m.get("label_lang", default_lang),
                language_v=m.get("value_lang", default_lang),
            )

    if providers:
        # https://iiif.io/api/cookbook/recipe/0234-provider/
        # https://github.com/giacomomarchioro/pyIIIFpres/blob/main/examples/0234-provider.py
        # TODO: set more provider fields eg homepage, logo, seeAlso
        for p in providers:
            prov = manifest.add_provider()
            prov.set_id(p.get("id"))
            prov.set_type()
            for label in p["labels"]:
                prov.add_label(
                    language=label.get("lang", default_lang), text=label.get("value")
                )

    if summary:
        if isinstance(summary, str):
            manifest.add_summary(language=default_lang, text=summary)
        else:
            for s in summary:
                manifest.add_summary(
                    language=s.get("lang", default_lang), text=s.get("value")
                )

    if thumbnails:
        for t in thumbnails:
            thum = manifest.add_thumbnail()
            thum.set_id(t.get("id"))
            thum.set_type(t.get("type", "Image"))
            thum.set_format(t.get("format", "image/jpeg"))
            thum.set_height(t.get("height"))
            thum.set_width(t.get("width"))

    for idx, d in enumerate(canvases):
        idx += 1
        canvas = manifest.add_canvas_to_items()
        canvas.set_id(extendbase_url=f"canvas/canvas:{ d.get('asset_id') }")
        canvas.set_height(d["height"])
        canvas.set_width(d["width"])
        if type(d["label"]) is dict:
            canvas.add_label(
                d["label"].get("lang", "default_lang"), d["label"].get("value")
            )
        else:
            canvas.add_label(default_lang, d["label"])
        if "metadata" in d:
            for m in d["metadata"]:
                canvas.add_metadata(
                    label=m["label"],
                    value=m["value"],
                    language_l=m.get("label_lang", default_lang),
                    language_v=m.get("value_lang", default_lang),
                )

        annopage = canvas.add_annotationpage_to_items()
        annopage.set_id(extendbase_url=f"annotationPage/annopage:{d.get('asset_id')}")
        annotation = annopage.add_annotation_to_items(
            target=canvas.id
        )  # TODO: think about handling multiple annotations (images) per canvas
        annotation.set_id(extendbase_url=f"annotation/annotation:{d.get('asset_id')}")
        annotation.set_motivation("painting")
        annotation.body.set_id(f"{d.get('id')}{d.get('service')}")
        annotation.body.set_type("Image")
        annotation.body.set_format(d.get("format"))
        annotation.body.set_width(d.get("width"))
        annotation.body.set_height(d.get("height"))
        s = annotation.body.add_service()
        s.set_id(d.get("id"))
        s.set_type(service_type)
        s.set_profile(service_profile)

    # TODO: Validate the manifest here before returning
    # slightly hacky - pyIIIFpres wants either the objid or extendbase_url to end in "/", but throws an error even if I add extendbase_url="/"
    # we want an ID without the trailing slash, so pass base_url without it; add the "/" manually in line 27; then set manifest.id back to no slash here
    manifest.id = base_url
    return manifest


def validateManifest(manifest: iiifpapi3.Manifest(), read_from_file: bool = True):
    """Validates a manifest. If you pass a string and read_from_file to True, it will read the JSON file to validate"""
    if not os.path.exists('iiif_3_0.json'):
        import urllib.request

        jsonchemadownloadurl = r"https://raw.githubusercontent.com/IIIF/presentation-validator/master/schema/iiif_3_0.json"
        with urllib.request.urlopen(jsonchemadownloadurl) as response, open(
            "iiif_3_0.json", 'wb'
        ) as out_file:
            data = response.read()
            out_file.write(data)
    if read_from_file:
        with open(manifest) as instance, open("iiif_3_0.json") as schema:
            jsonschema.validate(instance=json.load(instance), schema=json.load(schema))
    else:
        with open("iiif_3_0.json") as schema:
            jsonschema.validate(instance=json.load(instance), schema=json.load(schema))


# TODO move tests to an actual tests file, generate test manifests and read in and compare


def cookbook_test():
    manifest = createManifest(
        base_url="https://iiif.io/api/cookbook/recipe/0009-book-1/",
        labels=[{"lang": "en", "label": "Simple Manifest - Book"}],
        canvases=[
            # need to add asset_id for canvas_id
            # need to add format
            {
                "label": "Blank page",
                "width": 3204,
                "height": 4613,
                "id": "https://iiif.io/api/image/3.0/example/reference/59d09e6773341f28ea166e9f3c1e674f-gallica_ark_12148_bpt6k1526005v_f18",  # this is really the annotation.id - MPS asset location
                "service": "/full/max/0/default.jpg",
            },
            {
                "label": "Frontispiece",
                "width": 3186,
                "height": 4612,
                "id": "https://iiif.io/api/image/3.0/example/reference/59d09e6773341f28ea166e9f3c1e674f-gallica_ark_12148_bpt6k1526005v_f19",
                "service": "/full/max/0/default.jpg",
                "metadata": [
                    {
                        "label": "Reference",
                        "value": "ID124",
                        "label_lang": "en",
                        "value_lang": "en",
                    }
                ],
            },
        ],
        behaviors=["paged"],
        rights="http://creativecommons.org/licenses/by-sa/3.0/",
        required_statement=[
            {
                "label": "Attribution",
                "value": "Cole Crawford",
                "label_lang": "en",
                "value_lang": "en",
            }
        ],
        manifest_metadata=[
            {
                "label": "Creator",
                "value": "Unknown",
                "label_lang": "en",
                "value_lang": "en",
            },
            {
                "label": "Date",
                "value": "19th Century",
                "label_lang": "en",
                "value_lang": "en",
            },
        ],
    )
    manifest.json_save("test-manifest.json")
    return manifest


def mps_mcih_ingest_test():
    manifest = createManifest(
        base_url="https://nrs-qa.lib.harvard.edu/URN-3:AT:TESTMANIFEST3:MANIFEST:3",
        labels=[{"lang": "en", "label": "Test DARTH manifest - Cole"}],
        required_statement=[
            {
                "label": "Attribution",
                "value": "Jinah Kim",
                "label_lang": "en",
                "value_lang": "en",
            }
        ],
        canvases=[
            {
                "label": "Test image 1",  # this can be a string, which uses the default_lang, or a dict with lang/value
                "height": 564,
                "width": 3600,
                "asset_id": "TESTASSET3",  # canvas.id
                "id": "https://mps-qa.lib.harvard.edu/assets/images/AT:TESTASSET3",  # service.id
                "service": "/full/max/0/default.tif",  # anno.id = service.id + service; should have a default service based on format
                "format": "image/tiff",  # could get this from the filepath
                "metadata": [],
            },
            {
                "label": "Test image 2",
                "height": 581,
                "width": 3600,
                "asset_id": "TESTASSET4",
                "id": "https://mps-qa.lib.harvard.edu/assets/images/AT:TESTASSET4",
                "service": "/full/max/0/default.tif",
                "format": "image/tiff",
                "metadata": [],
            },
        ],
        rights="http://creativecommons.org/licenses/by-sa/3.0/",
        manifest_metadata=[
            {
                "label": "Creator",
                "value": "Unknown",
                "label_lang": "en",
                "value_lang": "en",
            }
        ],
    )
    manifest.json_save("ingest-test.json")
    return manifest


if __name__ == '__main__':
    mps_mcih_ingest_test()



import mimetypes
import os
from typing import Optional

import shortuuid
from PIL import Image

from .bucket import upload_image_get_metadata


def get_image_size(filepath):
    with Image.open(filepath) as img:
        w, h = img.size
        return w, h


def get_filename_noext(filepath):
    path_root = os.path.splitext(filepath)[0]
    return os.path.basename(path_root)


def create_asset_id(
    asset_prefix: str = "",
    identifier: str = "",
    with_uuid: bool = True,
):
    identifier = identifier if identifier else ""
    optional_uuid = shortuuid.uuid() if with_uuid else ""
    return f"{asset_prefix}{identifier}{optional_uuid}"


class Asset:
    """
    Constructs an Asset to be ingested.
    """

    def __init__(
        self,
        asset_id=None,
        filepath=None,
        s3key=None,
        format=None,
        extension=None,
        width=None,
        height=None,
        label=None,
        metadata=None,
    ):
        if asset_id and not asset_id.isalnum():
            raise ValueError(
                f"Invalid asset_id {asset_id} - must be alphanumeric only."
            )

        self.asset_id = asset_id
        self.filepath = filepath
        self.s3key = s3key
        self.format = format
        self.extension = extension
        self.width = width
        self.height = height
        self.label = label if label else ""
        self.metadata = metadata if metadata else {}

    def upload(
        self, bucket_name: str = "", s3_path: Optional[str] = None, boto_session=None
    ) -> str:
        """
        Uploads the asset to the designated bucket.
        """
        self.s3key = upload_image_get_metadata(
            image_path=self.filepath,
            bucket_name=bucket_name,
            s3_path=s3_path,
            session=boto_session,
        )
        return self.s3key

    @classmethod
    def from_file(cls, filepath, **kwargs):
        """
        Constructs an Asset from a file.
        """
        asset_id = kwargs.get("asset_id")

        if kwargs.get("width") and kwargs.get("height"):
            width = kwargs["width"]
            height = kwargs["height"]
        else:
            width, height = get_image_size(filepath)

        if kwargs.get("format"):
            format = kwargs.get("format")
        else:
            format, encoding = mimetypes.guess_type(filepath)

        if kwargs.get("extension"):
            extension = kwargs.get("extension")
        else:
            extension = mimetypes.guess_extension(format) or ""

        if kwargs.get("label"):
            label = kwargs.get("label")
        else:
            label = asset_id

        metadata = kwargs.get("metadata")

        return cls(
            filepath=filepath,
            asset_id=asset_id,
            format=format,
            extension=extension,
            width=width,
            height=height,
            label=label,
            metadata=metadata,
        )

    def to_dict(self):
        """
        Returns a dict representation of the Asset.
        """
        return {
            "asset_id": self.asset_id,
            "filepath": self.filepath,
            "s3key": self.s3key,
            "format": self.format,
            "extension": self.extension,
            "width": self.width,
            "height": self.height,
            "label": self.label,
            "metadata": self.metadata,
        }

    def __str__(self):
        return "Asset: " + str(sorted(self.to_dict().items()))



import argparse
import logging
import os

import boto3
from botocore.exceptions import ClientError

logger = logging.getLogger(__name__)


def upload_image_get_metadata(image_path, bucket_name, s3_path="", session=None):
    if not session:
        session = boto3._get_default_session()
    s3 = session.resource('s3')
    bucket = s3.Bucket(bucket_name)

    file_dir, file_name = os.path.split(image_path)
    # make sure s3_path ends in a slash
    if s3_path and not s3_path.endswith("/"):
        s3_path += "/"

    # extend to handle in-memory later, once we have file upload on an application
    # https://thecodinginterface.com/blog/aws-s3-python-boto3/
    # bytes_data =
    # obj = s3.Object(bucket_name, f"{s3_path}{file_name}")
    # obj.put(Body=bytes_data)

    key = f"{s3_path}{file_name}" if s3_path else file_name

    # try to upload it
    try:
        bucket.upload_file(Filename=image_path, Key=key)
        return key
    except ClientError as e:
        logging.error(e)
        return False


def upload_directory(path, bucket_name, session=None):
    if not session:
        session = boto3._get_default_session()
    s3 = session.resource('s3')
    bucket = s3.Bucket(bucket_name)

    for subdir, dirs, files in os.walk(path):
        for file in files:
            full_path = os.path.join(subdir, file)
            with open(full_path, 'rb') as data:
                bucket.put_object(Key=full_path[len(path) + 1 :], Body=data)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--dir", "-d", help="set directory")
    parser.add_argument("--bucket", "-b", help="set bucket")
    parser.add_argument("--file", "-f", help="set single file for upload")
    parser.add_argument("--s3path", help="Optional S3 path")
    args = parser.parse_args()

    if not args.bucket:
        print("Please set a bucket which you can use in the current AWS session")

    if args.dir and args.bucket:
        upload_directory(args.dir, args.bucket)

    if args.file and args.bucket:
        response = upload_image_get_metadata(
            args.file, args.bucket, s3_path=args.s3path
        )
        print(response)



import logging
import os
from datetime import datetime, timedelta
from zoneinfo import ZoneInfo

import jwt

from .settings import ROOT_DIR

logger = logging.getLogger(__name__)


class Credentials:
    def __init__(
        self,
        issuer: str = None,
        kid: str = None,
        resources: list = ["ingest"],
        algorithm: str = "RS256",
        expiration: int = 3600,
        timezone: str = "America/New_York",
        private_key_path: str = None,
        private_key_string: str = None,
    ):
        """
        issuer: the service which issued the token. Example: 'atdarth'
        private_key_path: path to the secret key provided by LTS for the given issuer
        private_key: stringified secret key
        ** either private_key_path or private_key is required. If both are provided, a conflict will be logged and the stringified version will be used**

        kid: key ID. Example: 'atdarthdefault'. A warning is logged if this is not passed, and the pattern f"{issuer}default" used
        resources: a list of resource types that the token is allowed to access
        expiration: length of time in seconds for which the token should be valid
        timezone: defaults to East Coast time
        algorithm: defaults to RS256
        """
        if not issuer:
            env_issuer = os.environ.get("LTS_IIIF_ISSUER")
            if env_issuer:
                logger.info("No issuer passed, using environment variable")
                self.issuer = env_issuer
            else:
                raise ValueError(
                    "No parameter passed or environment variable set for `LTS_IIIF_ISSUER`"
                )
        else:
            self.issuer = issuer

        if kid:
            self.kid = kid
        elif os.environ.get("LTS_IIIF_KID"):
            logger.info("No `kid` passed, using environment variable")
            self.kid = os.environ.get("LTS_IIIF_KID")
        else:
            self.kid = f"{self.issuer}default"
            logger.warning(f"No kid set, using {self.issuer}default")

        private_key_path_env = os.environ.get("LTS_IIIF_PRIVATE_KEY_PATH")
        private_key_string_env = os.environ.get("LTS_IIIF_PRIVATE_KEY_STRING")
        if (private_key_path_env or private_key_string_env) and (
            private_key_path or private_key_string
        ):
            logger.warning(
                "Environment variables present and args set; defaulting to args"
            )
        if private_key_path and private_key_string:
            logger.warning(
                "Both `private_key_path` and `private_key_string` set as params. Defaulting to `private_key_string`"
            )
            self.private_key = private_key_string
        elif private_key_string:
            self.private_key = private_key_string
        elif private_key_path:
            self.private_key = open(private_key_path, "r").read()
        elif private_key_string_env:
            self.private_key = private_key_string_env
        elif private_key_path_env:
            self.private_key = open(private_key_path_env, "r").read()
        else:
            raise ValueError(
                "No args (`private_key_path` or `private_key_string`) provided. Neither `LTS_IIIF_PRIVATE_KEY_PATH` nor `LTS_IIIF_PRIVATE_KEY_STRING` are set."
            )

        valid_resources = ["ingest", "content", "description"]
        for r in resources:
            if r not in valid_resources:
                raise ValueError("Invalid resource type provided")
        self.resources = resources

        self.algorithm = algorithm
        self.expiration = expiration
        self.timezone = timezone

    def make_jwt(
        self,
        resources: list = None,
        algorithm: str = None,
        expiration: int = None,
        timezone: str = None,
    ):
        """
        Makes a JWT token
        Example token data
        Header
            {
                "alg": "RS256",
                "typ": "JWT",
                "iss": "testissuer",
                "kid": "testkeyid",
                "resources": [
                    "description"
                ]
            }

        Payload
        {
            "iat": 1619628554,
            "exp": 3239257168
        }

        Valid algs: RS256, ???
        Exp range: <8 hours maximum

        """
        logger.info("Making IIIF LTS jwt")
        if not resources:
            resources = self.resources
        if not algorithm:
            algorithm = self.algorithm
        if not expiration:
            expiration = self.expiration
        if not timezone:
            timezone = self.timezone

        timestamp = datetime.now(ZoneInfo(timezone))
        header = {
            "typ": "JWT",
            "alg": algorithm,
            "iss": self.issuer,
            "kid": self.kid,
            "resources": resources,
        }
        payload = {"iat": timestamp, "exp": timestamp + timedelta(seconds=expiration)}

        encoded_jwt = jwt.encode(
            payload, self.private_key, algorithm=algorithm, headers=header
        )
        return encoded_jwt


def test():
    creds = Credentials(
        issuer="atdarth",
        resources=["ingest"],
        kid="atdarthdefault",
        private_key_path=os.path.join(
            ROOT_DIR, "auth/qa/keys/atdarth/atdarthdefault/private.key"
        ),
    )
    print(creds)
    jwt = creds.make_jwt()
    print(jwt)


if __name__ == '__main__':
    test()



import json
import logging
import os
from typing import List, Optional

import requests
import shortuuid

from .asset import Asset, create_asset_id
from .generate_manifest import createManifest
from .ingest import createImageAsset, pingJob, sendIngestRequest, wrapIngestRequest
from .settings import (
    MPS_ASSET_BASE_URL,
    MPS_ASSET_BASE_URL_PROD,
    MPS_BUCKET_NAME,
    MPS_DEV_INGEST_SERVICE_STATUS,
    MPS_INGEST_ENDPOINT_PRIVATE,
    MPS_INGEST_ENDPOINT_PROD,
    MPS_INGEST_ENDPOINT_QA,
    MPS_JOBSTATUS_ENDPOINT_PRIVATE,
    MPS_JOBSTATUS_ENDPOINT_PROD,
    MPS_JOBSTATUS_ENDPOINT_QA,
    MPS_MANIFEST_BASE_URL,
    MPS_MANIFEST_BASE_URL_PROD,
    MPS_PROD_INGEST_SERVICE_STATUS,
    MPS_QA_INGEST_SERVICE_STATUS,
    VALID_ENVIRONMENTS,
)

logger = logging.getLogger(__name__)


class Client:
    """
    Constructs the ingest API client.
    """

    def __init__(
        self,
        account: str = "at",
        space: str = "atdarth",
        namespace: str = "at",
        asset_prefix: str = "",
        agent: str = "atagent",
        environment: str = "qa",
        jwt_creds=None,
        boto_session=None,
    ):
        if not namespace or not namespace.isalnum():
            raise ValueError("Invalid or missing namespace_prefix")
        if environment not in VALID_ENVIRONMENTS:
            raise ValueError(
                f"Invalid environment: {environment} must be one of: {VALID_ENVIRONMENTS}"
            )

        namespace = namespace.upper()

        self.account = account
        self.space = space
        self.namespace = namespace
        self.asset_prefix = asset_prefix
        self.agent = agent
        self.environment = environment
        self.jwt_creds = jwt_creds
        self.boto_session = boto_session

        self.bucket_name = MPS_BUCKET_NAME.format(
            account=account, space=space, environment=environment
        )

        # Some divergence in dev/qa/prod ingest endpoint formats; prod doesn't have prod in URL, QA/Prod admin endpoint is different format than dev
        if self.environment == "dev":
            self.asset_base_url = MPS_ASSET_BASE_URL.format(
                environment=environment, namespace=namespace
            )
            self.ingest_endpoint = MPS_INGEST_ENDPOINT_PRIVATE.format(
                environment=environment
            )
            self.job_endpoint = MPS_JOBSTATUS_ENDPOINT_PRIVATE.format(
                environment=environment
            )
            self.manifest_base_url = MPS_MANIFEST_BASE_URL.format(
                environment=environment, namespace=namespace
            )
            self.ingest_service_status_endpoint = MPS_DEV_INGEST_SERVICE_STATUS
        elif self.environment == "qa":
            self.asset_base_url = MPS_ASSET_BASE_URL.format(
                environment=environment, namespace=namespace
            )
            self.ingest_endpoint = MPS_INGEST_ENDPOINT_QA
            self.job_endpoint = MPS_JOBSTATUS_ENDPOINT_QA
            self.manifest_base_url = MPS_MANIFEST_BASE_URL.format(
                environment=environment, namespace=namespace
            )
            self.ingest_service_status_endpoint = MPS_QA_INGEST_SERVICE_STATUS
        elif self.environment == "prod":
            self.asset_base_url = MPS_ASSET_BASE_URL_PROD.format(namespace=namespace)
            self.ingest_endpoint = MPS_INGEST_ENDPOINT_PROD
            self.job_endpoint = MPS_JOBSTATUS_ENDPOINT_PROD
            self.manifest_base_url = MPS_MANIFEST_BASE_URL_PROD.format(
                namespace=namespace
            )
            self.ingest_service_status_endpoint = MPS_PROD_INGEST_SERVICE_STATUS

    def _get_asset_url(self, asset_id: str) -> str:
        """Constructs the asset URL."""
        return f"{self.asset_base_url}{asset_id}"

    def _get_manifest_url(self, manifest_name: str, prezi_version: int = 3) -> str:
        """Constructs the manifest URL."""
        return f"{self.manifest_base_url}{manifest_name}:MANIFEST:{prezi_version}"

    def upload(self, images: List[dict], s3_path: str = "") -> List[Asset]:
        """
        Uploads a list of images to the MPS ingest bucket in S3.
        Returns a list of assets.
        """
        logger.debug(f"Uploading {len(images)} images")
        assets = []
        for image in images:
            filepath = image["filepath"]
            asset_id = create_asset_id(
                asset_prefix=self.asset_prefix,
                identifier=image.get("id"),
            )
            asset = Asset.from_file(
                filepath, asset_id=asset_id, label=image.get("name")
            )
            asset.upload(
                bucket_name=self.bucket_name,
                s3_path=s3_path,
                boto_session=self.boto_session,
            )
            assets.append(asset)
        logger.debug(f"Upload completed. Returning assets: {assets}")
        return assets

    def create_manifest(
        self,
        manifest_level_metadata: dict,
        assets: List[Asset],
        manifest_name: str = "",
        prezi_version: int = 3,
    ) -> dict:
        """
        Creates a manifest from manifest level metadata and a list of image assets.
        Returns the created manifest as a dict.
        """
        if not manifest_name:
            manifest_name = f"GEN{shortuuid.uuid()}"

        canvases = []
        for asset in assets:
            canvas = asset.to_dict()
            canvas["id"] = self._get_asset_url(asset.asset_id)
            canvas["service"] = f"/full/max/0/default{asset.extension}"
            canvases.append(canvas)

        logger.debug(f"Creating manifest from data: {manifest_level_metadata}")
        manifest_kwargs = dict(
            base_url=self._get_manifest_url(
                manifest_name=manifest_name, prezi_version=prezi_version
            ),
            canvases=canvases,
            labels=manifest_level_metadata.get("labels", None),
            behaviors=manifest_level_metadata.get("behaviors", None),
            providers=manifest_level_metadata.get("providers", None),
            rights=manifest_level_metadata.get("rights", None),
            required_statement=manifest_level_metadata.get("required_statement", None),
            manifest_metadata=manifest_level_metadata.get("metadata", None),
            default_lang=manifest_level_metadata.get(
                "default_lang", None
            ),  # need to add this?
            service_type=manifest_level_metadata.get(
                "service_type", None
            ),  # need to add this ?
            service_profile=manifest_level_metadata.get(
                "service_profile", None
            ),  # need to add this
        )
        manifest = createManifest(
            **{k: v for k, v in manifest_kwargs.items() if v is not None}
        )
        manifest_json = manifest.json_dumps()
        logger.debug(f"Created manifest: {manifest_json}")

        manifest_dict = json.loads(manifest_json)
        return manifest_dict

    def ingest(
        self,
        assets: List[Asset],
        manifest: Optional[dict] = None,
        policy_definition: Optional[dict] = None,
    ) -> dict:
        """
        Sends ingest request for assets and manifest.
        Returns the job ID.
        """
        if manifest is None:
            manifest = {}

        logger.debug(f"Preparing {len(assets)} ingest assets")
        ingest_assets = []
        for asset in assets:
            src_path, src_key = os.path.split(asset.s3key)
            params = {
                "space": self.space,
                "createdByAgent": self.agent,
                "identifier": f"{self.namespace}:{asset.asset_id}",
                "storageSrcPath": f"{src_path}/",
                "storageSrcKey": src_key,
                "policyDefinition": policy_definition,
                "assetMetadata": [
                    {
                        "fieldName": "imageSize",
                        "jsonValue": {
                            "width": asset.width,
                            "height": asset.height,
                        },
                    }
                ],
            }
            ingest_asset = createImageAsset(
                **{k: v for k, v in params.items() if v is not None}
            )
            ingest_assets.append(ingest_asset)

        token = self.jwt_creds.make_jwt()
        logger.debug(f"Generated ingest auth token: {token}")

        request_body = wrapIngestRequest(
            assets=ingest_assets,
            manifest=manifest,
            metadata={},
            space_default=self.space,
            action_default="upsert",
        )

        logger.debug(f"Sending ingest request: {request_body}")
        response = sendIngestRequest(
            req=request_body,
            endpoint=self.ingest_endpoint,
            token=token,
        )
        logger.debug(
            f"Received ingest response: {response.status_code} {response.text}"
        )
        response_data = response.json()

        job_id = (
            response_data.get("data", {}).get("job_tracker_file", {}).get("_id", "")
        )
        if not job_id:
            logger.warning("Ingest job ID not found. Maybe the ingest request failed?")
        else:
            logger.info(f"Ingest job ID: {job_id}")

        result = {
            "job_id": str(job_id),
            "error": response_data.get("error", None),
            "data": response_data.get("data", {}),
        }
        return result

    def jobstatus(self, job_id: str) -> dict:
        """
        Returns the status of an ingest request.
        """
        logger.info(f"Pinging job {job_id}")
        status = pingJob(
            job_id=job_id,
            endpoint=self.job_endpoint,
        )
        logger.info(f"Job status: {status}")

        if status.get("completed"):
            logger.debug("Job completed.")
        else:
            logger.debug("Job failed or did not complete in the allotted timeframe")

        return status

    def servicestatus(self) -> bool:
        """
        Returns whether the MPS ingest service is up or down
        """
        logger.info(f"Pinging service {self.ingest_service_status_endpoint}")
        r = requests.get(self.ingest_service_status_endpoint)
        return r.status_code == 200

    def __repr__(self):
        return (
            f'{self.__class__.__name__}('
            f'{self.account!r}, {self.space!r}, {self.namespace!r}, {self.agent!r}, {self.asset_prefix!r})'
        )



import logging
import time
from datetime import datetime
from urllib import request
from zoneinfo import ZoneInfo

import requests

logger = logging.getLogger(__name__)


# For consistency, either imageAsset should also be a class, or turn IIIFCanvas into a method which wraps dict properties
# Currently, generate_manifest expects a list of dicts
def createImageAsset(
    identifier: str,
    space: str,
    storageSrcPath: str,
    storageSrcKey: str,
    action: str = "create",
    createdByAgent: str = "atagent",
    lastModifiedByAgent: str = "atagent",
    createDate: str = datetime.now(ZoneInfo("America/New_York")).strftime(
        "%Y-%m-%d %H:%M:%S"
    ),
    lastModifiedDate: str = datetime.now(ZoneInfo("America/New_York")).strftime(
        "%Y-%m-%d %H:%M:%S"
    ),
    status: str = "ACTIVE",
    iiifApiVersion: str = "3",
    policyDefinition: dict = {"policyGroupName": "default"},
    assetMetadata: list = [],
) -> dict:
    return {
        "action": action,
        "storageSrcPath": storageSrcPath,
        "storageSrcKey": storageSrcKey,
        "identifier": identifier,
        "space": space,
        "createdByAgent": createdByAgent,
        "createDate": createDate,
        "lastModifiedByAgent": lastModifiedByAgent,
        "lastModifiedDate": lastModifiedDate,
        "status": status,
        "iiifApiVersion": iiifApiVersion,
        "policyDefinition": policyDefinition,
        "assetMetadata": assetMetadata,
    }


def wrapIngestRequest(
    metadata: dict,
    assets: list,
    manifest: dict,
    space_default: str,
    action_default: str = "upsert",
) -> dict:
    """Creates an ingest request to be sent to the LTS MPS ingest API"""
    req = {
        "globalSettings": {
            "actionDefault": action_default,
            "spaceDefault": space_default,
        },
        "metadata": metadata,
        "assets": {"audio": [], "video": [], "text": [], "image": assets},
        "manifest": manifest,
    }
    return req


def sendIngestRequest(req: dict, endpoint: str, token) -> request:
    r = requests.post(endpoint, headers={"Authorization": f"Bearer {token}"}, json=req)
    return r


def jobStatus(
    job_id: str,
    endpoint: str = "https://mps-admin-qa.lib.harvard.edu/admin/ingest/jobstatus/",
) -> request:
    url = f"{endpoint}{job_id}"
    r = requests.get(url)
    return r


def pingJob(
    job_id: str,
    endpoint: str = "https://mps-admin-qa.lib.harvard.edu/admin/ingest/jobstatus/",
    max_pings: int = 25,
    interval: int = 10,
) -> dict:
    working = True
    completed = False
    start = time.time()
    pings = 0
    status = {}
    while working:
        pings += 1
        r = jobStatus(job_id, endpoint)
        status = r.json()
        end = time.time()
        msg = ""
        if status["data"].get("job_status") == "success":
            msg = f"------- Job {job_id} finished ingesting after {round(end - start)} seconds and {pings} pings -------"
            logger.debug(msg)
            logger.debug(status)
            completed = True
            working = False
        elif status["data"].get("job_status") == "running":
            msg = f"-------- Job {job_id} processing. {round(end - start)} seconds and {pings} pings -------"
            logger.debug(msg)
            logger.debug(status)
            time.sleep(interval)
        elif status["data"].get("job_status") == "failed":
            msg = f"-------- Job {job_id} Failed. {round(end - start)} seconds and {pings} pings -------"
            logger.debug(msg)
            logger.debug(status)
            working = False
        elif status["data"].get("job_status") == "queued":
            msg = f"-------- Job {job_id} queued. {round(end - start)} seconds and {pings} pings -------"
            logger.debug(msg)
            logger.debug(status)
            time.sleep(interval)
        elif pings > max_pings:
            msg = f"-------- Job {job_id} did not complete within {round(end - start)} seconds and {pings} pings (max pings {max_pings})"
            logger.debug(status)
            working = False
        else:
            msg = f"-------- Job {job_id} delivered an invalid status. {round(end - start)} seconds and {pings} pings -------"
            logger.debug(status)
            working = False

    return {
        "completed": completed,
        "message": msg,
        "job_id": job_id,
        "endpoint": endpoint,
        "pings": pings,
        "elapsed": round(time.time() - start),
        "job_status": status["data"].get("job_status"),
    }
